% Generate z_sgm and w_sgm, using SGM implementation from 
% Yamaguchi et al., 2014
%
% Copyright (c) 2014, Ayan Chakrabarti <ayanc@ttic.edu>
function [z,w] = dosgm(imgid)

global dlstr drstr SGMEXEC


% Call Yamaguchi code.
lname = sprintf(dlstr,imgid);
rname = sprintf(drstr,imgid);

img = imread(lname);
[h w] = size(img);

dat = tempname;

system([SGMEXEC ' ' lname ' ' rname ' ' dat]);
fp = fopen(dat,'rb');
z = fread(fp,[w h],'float')';
fclose(fp);

delete(dat);

% Set weights
[gx,gy] = gradient(z);
mp = conv2(double(abs(gx) > 1 & abs(gy) > 1),ones(3,3),'same') > 0;
w = double(z > 0); 
w(mp == 1 & w == 1) = 0.25;

% Fill in w = 0 values to provide decent initialization.
z = fillH(-z,z > 0);

% Fill in holes in disparity map
function dout = fillH(din,vld)

dl = din; dr = din; du = din; dd = din;
vl = vld; vr = vld; vu = vld; vd = vld;

for i = 2:size(din,2)
  vl(vld(:,i) == 0,i)=vl(vld(:,i) == 0,i-1);
  dl(vld(:,i) == 0,i)=dl(vld(:,i) == 0,i-1).*vl(vld(:,i) == 0,i-1);
end;

for i = size(din,2)-1:-1:1
  vr(vld(:,i) == 0,i)=vr(vld(:,i) == 0,i+1);
  dr(vld(:,i) == 0,i)=dr(vld(:,i) == 0,i+1).*vr(vld(:,i) == 0,i+1);
end;

for i = 2:size(din,1)
  vu(i,vld(i,:) == 0)=vu(i-1,vld(i,:) == 0);
  du(i,vld(i,:) == 0)=du(i-1,vld(i,:) == 0).*vu(i-1,vld(i,:) == 0);
end;

for i = size(din,1)-1:-1:1
  vd(i,vld(i,:) == 0)=vd(i+1,vld(i,:) == 0);
  dd(i,vld(i,:) == 0)=dd(i+1,vld(i,:) == 0).*vd(i+1,vld(i,:) == 0);
end;

dA = sort([dl(:) dr(:) du(:) dd(:)],2);
nv = vl(:)+vr(:)+vu(:)+vd(:);

dout = zeros(size(din));
dout(nv == 1) = dA(nv == 1,1);
dout(nv == 2) = 0.5*(dA(nv == 2,1)+dA(nv == 2,2));
dout(nv == 3) = dA(nv == 3,2);
dout(nv == 4) = 0.5*(dA(nv == 4,2)+dA(nv == 4,3));

dout(vld == 1) = din(vld == 1);
