% getIMaps: Compute intensity based outlier cost factors
%
% function imaps = getIMaps(imgid,minlev,maxlev)
%
% Copyright (C) 2014, Ayan Chakrabarti <ayanc@ttic.edu>
function imaps = getIMaps(imgid,minlev,maxlev)

thresh = 0.5;
tol = 1.5;

psizes = 2.^[minlev:maxlev];
global dlstr
img = double(imread(sprintf(dlstr,imgid)));
img = img.^2;

U = ones(size(img));
pv = upsweep(img,img.^2,U,minlev,maxlev);

for i = 1:length(psizes)
  psz = psizes(i); 
  
  pvar = pv(1:end-psz+1,1:end-psz+1,2,i)/psz.^2 - (pv(1:end-psz+1,1:end-psz+1,1,i)/psz.^2).^2;
  imi = zeros(size(pvar),'uint8');
  
  % Check to see how many of our neighbors we're worse than
  for dx = [-1 0 1]*psz/2
    for dy = [-1 0 1]*psz/2
      if dx == 0 && dy == 0
	continue;
      end;
      imi(1+max(-dy,0):end-max(dy,0),1+max(-dx,0):end-max(dx,0)) = ...
	  imi(1+max(-dy,0):end-max(dy,0),1+max(-dx,0):end-max(dx,0))+ ...
	  uint8(pvar(1+max(-dy,0):end-max(dy,0),1+max(-dx,0):end-max(dx,0)) ...
	  > pvar(1+max(dy,0):end-max(-dy,0),1+max(dx,0):end-max(-dx,0))*tol);
      
    end;
  end;
  imaps{i} = max(thresh,2.^(-double(imi).^2/4));
end;
  