/* 
 Copyright (C) 2014, Ayan Chakrabarti <ayanc@ttic.edu>
*/

#include "mex.h"
#include <omp.h>
#include <string.h>
#include "libPyr.h"

void * env_alloc(size_t sz) {
	return mxMalloc(sz);
}

void env_free(void * ptr) {
	mxFree(ptr);
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	const mwSize * dims;
	mwSize odims[4];
	double * Fout, *F1, *F2, lambda;
	int w,h,M;
	int minlev, maxlev;
	int numt=1;
	

	if(mexGetVariablePtr("global","NCORES"))
		numt = mxGetScalar(mexGetVariablePtr("global","NCORES"));
	omp_set_num_threads(numt);


	dims = mxGetDimensions(prhs[0]);
	h = dims[0]; w = dims[1];

	F1 = mxGetPr(prhs[0]); F2 = mxGetPr(prhs[1]); lambda = mxGetScalar(prhs[2]);

	M = mxGetScalar(prhs[3]); 
	minlev = mxGetScalar(prhs[4]); maxlev = mxGetScalar(prhs[5]);

	odims[0] = h; odims[1] = w; odims[2] = dims[2]; odims[3] = maxlev-minlev+1;
	plhs[0] = mxCreateNumericArray(4, odims, mxDOUBLE_CLASS, mxREAL);
	Fout = mxGetPr(plhs[0]);

#pragma omp parallel 
	{
		int tid;
		tid = omp_get_thread_num();


		block_sumwt(w,h,M,Fout,F1,F2,lambda,minlev,maxlev,numt,tid);
		cholesky(w,h,M,Fout,minlev,maxlev,numt,tid);
	}
	
}
