/* 
 Copyright (C) 2014, Ayan Chakrabarti <ayanc@ttic.edu>
*/

#include "mex.h"
#include <omp.h>
#include "libPyr.h"

void * env_alloc(size_t sz) {
	return mxMalloc(sz);
}

void env_free(void * ptr) {
	mxFree(ptr);
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	const mwSize * dims;
	mwSize odims[3];
	double * Z, * theta, * U;
	uint16_t * J, * I;
	int w,h,M,d;
	int minlev, maxlev;
	int numt=1;

	if(mexGetVariablePtr("global","NCORES"))
		numt = mxGetScalar(mexGetVariablePtr("global","NCORES"));
	omp_set_num_threads(numt);


	dims = mxGetDimensions(prhs[2]);
	h = dims[0]; w = dims[1]; d = dims[2]; M = dims[3];

        theta = mxGetPr(prhs[0]); I = mxGetData(prhs[1]);
	U = mxGetPr(prhs[2]);
	minlev = mxGetScalar(prhs[3]); maxlev = mxGetScalar(prhs[4]);

	odims[0] = h; odims[1] = w; odims[2] = d;
	plhs[0] = mxCreateNumericArray(3, odims, mxDOUBLE_CLASS, mxREAL);
	plhs[1] = mxCreateNumericArray(2, odims, mxUINT16_CLASS, mxREAL);
        Z = mxGetPr(plhs[0]);
        J = mxGetData(plhs[1]);

#pragma omp parallel 
	{
		int tid;
		tid = omp_get_thread_num();

		downsweep(w,h,M,d,Z,J,minlev,maxlev,theta,I,U,numt,tid);
	}
	
}
