/* 
 Copyright (C) 2014, Ayan Chakrabarti <ayanc@ttic.edu>
*/

#include "mex.h"
#include <omp.h>
#include <stdint.h>
#include <string.h>
#include "libPyr.h"

void * env_alloc(size_t sz) {
	return mxMalloc(sz);
}

void env_free(void * ptr) {
	mxFree(ptr);
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	const mwSize * dims;
	
	double  * the, * imaps;
	double outcost;
	int w,h,M,lev;
	int minlev, maxlev, nlev;
	

	double * theta;
	double totcost = 0, *tcosts;
	uint16_t numout = 0, *nouts;
	uint16_t * I;

	int tid,numt=1;
	

	if(mexGetVariablePtr("global","NCORES"))
		numt = mxGetScalar(mexGetVariablePtr("global","NCORES"));
	omp_set_num_threads(numt);


	tcosts = env_alloc(numt*sizeof(double));
	nouts = env_alloc(numt*sizeof(uint16_t));

	dims = mxGetDimensions(prhs[0]);
	h = dims[0]; w = dims[1]; M = dims[2]-1;

	the = mxGetPr(prhs[0]); imaps = mxGetPr(prhs[1]);	
	outcost = mxGetScalar(prhs[2]);
	minlev = mxGetScalar(prhs[3]); maxlev = mxGetScalar(prhs[4]);
	nlev = maxlev-minlev+1;

	theta = mxGetPr(mexGetVariablePtr("caller",mxArrayToString(prhs[5])));
	I = mxGetData(mexGetVariablePtr("caller",mxArrayToString(prhs[6])));

	for(lev = 0; lev < nlev; lev++)
		memcpy(&theta[w*h*M*lev],&the[w*h*(M+1)*lev],w*h*M*sizeof(double));

#pragma omp parallel 
	{
		int tid;
		double * tcost;
		uint16_t * nout;
		int P,lev;

		tid = omp_get_thread_num();
		tcost = &tcosts[tid]; nout = &nouts[tid];
		*tcost = 0; *nout = 0;

		P = 1 << minlev;
		for(lev = 0; lev < nlev; lev++) {
			findout(w,h,M,&theta[w*h*M*lev],&I[w*h*lev],
				tcost,nout,&the[w*h*(M+1)*lev+w*h*M],&imaps[w*h*lev],
				outcost,P,numt,tid);
			P *= 2;
		}
	}
	
	for(tid = 0; tid < numt; tid++) {
		totcost += tcosts[tid];
		numout += nouts[tid];
	}
	
	plhs[0] = mxCreateDoubleScalar(numout);
	plhs[1] = mxCreateDoubleScalar(totcost);

	env_free(tcosts); env_free(nouts);
}
