/* 
 Routines for hierarchical computations in CoR.
 Copyright (C) 2014, Ayan Chakrabarti <ayanc@ttic.edu>
*/

#ifndef __LIBPYR_H
#define __LIBPYR_H

#include <stdint.h>

/* Build \theta, e pyramid from Z, Z^2 */
void upsweep(int w, int h, int M, int D, double * thetae,int minlev, int maxlev,double * Z, double * Z2, double * U, int tid, int numt);

/* Reconstruct Z, J from \theta, I pyramid */
void downsweep(int w, int h, int M, int D, double * Z, uint16_t * J, int minlev, int maxlev, double * theta, uint16_t * I, double * U, int numt, int tid);

/* Compute in-place cholesky decomposition of per-patch symmetric matrices */
void cholesky(int w, int h, int M, double * F0, int minlev, int maxlev, int numt, int tid);

/* Compute in-place minimization of an L2 cost (and compute the value of that cost) */
void minl2cost(int w, int h, int M, double * thetae, double * thetac, double * F0, int minlev, int maxlev, int numt, int tid);

/* Detect outliers and set corresponding theta to 0 */
void findout(int w, int h, int M, double * theta, uint16_t * I, double * tcost, uint16_t * nout, double * cost, double * imap, double ocost, int P, int numt, int tid);

/* Compute C = A + lambda * B */
void block_sumwt(int w, int h, int M, double * F0, double * F1, double * F2, double lambda, int minlev, int maxlev, int numt, int tid);

#endif
