/* 
 Copyright (C) 2014, Ayan Chakrabarti <ayanc@ttic.edu>
*/

#include "mex.h"
#include <omp.h>
#include <string.h>
#include "libPyr.h"

void * env_alloc(size_t sz) {
	return mxMalloc(sz);
}

void env_free(void * ptr) {
	mxFree(ptr);
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	const mwSize * dims;
	double * thc, * the, * F0;
	int w,h,M;
	int minlev, maxlev;
	int numt=1;
	

	if(mexGetVariablePtr("global","NCORES"))
		numt = mxGetScalar(mexGetVariablePtr("global","NCORES"));
	omp_set_num_threads(numt);


	dims = mxGetDimensions(prhs[0]);
	h = dims[0]; w = dims[1]; M = dims[2]-1;

	the = mxGetPr(prhs[0]); F0 = mxGetPr(prhs[1]);	
	minlev = mxGetScalar(prhs[2]); maxlev = mxGetScalar(prhs[3]);

	thc = mxGetPr(mexGetVariablePtr("caller",mxArrayToString(prhs[4])));
	memcpy(thc,the,w*h*(M+1)*(maxlev-minlev+1)*sizeof(double));

#pragma omp parallel 
	{
		int tid;
		tid = omp_get_thread_num();

		minl2cost(w,h,M,thc,the,F0,minlev,maxlev,numt,tid);
	}
	
}
