/* 
 Copyright (C) 2014, Ayan Chakrabarti <ayanc@ttic.edu>
*/

#include "mex.h"
#include <omp.h>
#include "libPyr.h"

void * env_alloc(size_t sz) {
	return mxMalloc(sz);
}

void env_free(void * ptr) {
	mxFree(ptr);
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	const mwSize * dims;
	mwSize odims[4], ndims;
	double * Z, *Z2, *U, *thetae;
	int w,h,M,d;
	int minlev, maxlev;
	int numt=1;
	
	if(mexGetVariablePtr("global","NCORES"))
		numt = mxGetScalar(mexGetVariablePtr("global","NCORES"));
	omp_set_num_threads(numt);


	ndims = mxGetNumberOfDimensions(prhs[2]);
	dims = mxGetDimensions(prhs[2]);
	h = dims[0]; w = dims[1]; 
	if(ndims > 2)
		d = dims[2]; 
	else
		d = 1;
	if(ndims > 3)
		M = dims[3];
	else
		M = 1;

	Z = mxGetPr(prhs[0]); Z2 = mxGetPr(prhs[1]); U = mxGetPr(prhs[2]);
	minlev = mxGetScalar(prhs[3]); maxlev = mxGetScalar(prhs[4]);

	odims[0] = h; odims[1] = w; odims[2] = M+1; odims[3] = maxlev-minlev+1;
	plhs[0] = mxCreateNumericArray(4, odims, mxDOUBLE_CLASS, mxREAL);
	thetae = mxGetPr(plhs[0]);

#pragma omp parallel 
	{
		int tid;
		tid = omp_get_thread_num();

		upsweep(w,h,M,d,thetae,minlev,maxlev,Z,Z2,U,numt,tid);
	}
	
}
