/* 
 Copyright (C) 2014, Ayan Chakrabarti <ayanc@ttic.edu>
*/

#include "mex.h"
#include <omp.h>
#include "libPyr.h"
#include "vecops.h"

void * env_alloc(size_t sz) {
	return mxMalloc(sz);
}

void env_free(void * ptr) {
	mxFree(ptr);
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {

	const mwSize * dims;
	double * Z, *Z2, *U, *thetae, *thsub=NULL;
	int w,h,M,d, ninv;
	int minlev, maxlev;
	int numt=1;
	
	if(mexGetVariablePtr("global","NCORES"))
		numt = mxGetScalar(mexGetVariablePtr("global","NCORES"));
	omp_set_num_threads(numt);


	dims = mxGetDimensions(prhs[2]);
	h = dims[0]; w = dims[1]; d = dims[2]; M = dims[3];

	Z = mxGetPr(prhs[0]); Z2 = mxGetPr(prhs[1]); U = mxGetPr(prhs[2]);
	minlev = mxGetScalar(prhs[3]); maxlev = mxGetScalar(prhs[4]);

	thetae = mxGetPr(mexGetVariablePtr("caller",mxArrayToString(prhs[5])));

	if(nrhs == 8) {
		thsub = mxGetPr(mexGetVariablePtr("caller",mxArrayToString(prhs[6])));
		ninv = mxGetScalar(prhs[7]);
	}

#pragma omp parallel 
	{
		int tid;
		tid = omp_get_thread_num();

		upsweep(w,h,M,d,thetae,minlev,maxlev,Z,Z2,U,numt,tid);
		if(thsub) {
			int ist, iend, i;
#pragma omp barrier
			ist = (tid *w*h)/numt; iend = ((tid+1)*w*h)/numt;
			ist *= (M+1)*ninv; iend *= (M+1)*ninv;
			
			vec_acc(&thetae[ist],&thsub[ist],iend-ist);

		}
	}
	
}
